//
//  MyScene.m
//  SpriteKitDemo1
//

#import "MyScene.h"

@implementation MyScene{
	
	BOOL		_markOnWing;
	BOOL		_numberOnBody;
	int			_actionNum;
}

-(id)initWithSize:(CGSize)size {
    if (self = [super initWithSize:size]) {
        /* Setup your scene here */
        
        self.backgroundColor = [SKColor colorWithRed:0.15 green:0.15 blue:0.3 alpha:1.0];
        
        SKLabelNode *myLabel = [SKLabelNode labelNodeWithFontNamed:@"Chalkduster"];
        
        myLabel.text = @"Hello, Sprite Kit!";
        myLabel.fontSize = 30;
        myLabel.position = CGPointMake(CGRectGetMidX(self.frame),
                                       CGRectGetMidY(self.frame));
        
        [self addChild:myLabel];
		
		
		// 子ノードの追加
		//_numberOnBody = YES;	//ラベルで数字を張る
		//_markOnWing = YES;		//星マーク画像を両翼に張る
		
		//アクションあれこれ
		_actionNum = 0;		//アクション指定番号（0〜17）
		
    }
    return self;
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    /* Called when a touch begins */
    
    for (UITouch *touch in touches) {
        
		SKSpriteNode *sprite;
		//画像からスプライトを作成
		sprite = [SKSpriteNode spriteNodeWithImageNamed:@"Spaceship"];
		sprite.name = @"Spaceship";
		CGPoint location = [touch locationInNode:self];
		sprite.position = location;	//タッチした位置にスプライトを配置
		//シーンに追加
		[self addChild:sprite];
		
		//------------------------------
		// 子ノードを追加する
		//------------------------------
		//ラベルで数字を張る
		if( _numberOnBody ){
			SKLabelNode*	num;
			
			num = [SKLabelNode labelNodeWithFontNamed:@"Copperplate-Bold"];
			[sprite addChild:num];
			num.text = @"99";
			num.fontSize = 50;
			num.fontColor = [UIColor blackColor];
		}
		//星マーク画像を両翼に張る
		if( _markOnWing ){
			
			SKSpriteNode*	num;
			//画像からスプライトを作成
			num = [SKSpriteNode spriteNodeWithImageNamed:@"star"];
			[sprite addChild:num];
			num.position = CGPointMake(-100.0, -60.0);
			num.name = @"star1";
			
			num = [SKSpriteNode spriteNodeWithImageNamed:@"star"];
			[sprite addChild:num];
			num.position = CGPointMake(100.0, -60.0);
			num.name = @"star2";
		}
		
		//------------------------------
		// アクションあれこれ
		//------------------------------
		if(_actionNum==0){
			//無限の回転（テンプレートオリジナル）
			SKAction *action = [SKAction rotateByAngle:M_PI duration:1];
			[sprite runAction:[SKAction repeatActionForever:action]];
		}
		if(_actionNum==1){
			//位置の移動
			//SKAction*	action = [SKAction moveByX:100 y:50 duration:1.0];
			SKAction*	action = [SKAction moveTo:CGPointMake(100, 50) duration:1.0];
			[sprite runAction:action];
		}
		if(_actionNum==2){
			//パス移動
			CGPathRef path = CGPathCreateWithRect(CGRectMake(20, 20, 100, 100), nil);
			SKAction*	action;
			action = [SKAction followPath:path
								 asOffset:NO	// YES=相対座標 NO=絶対座標
							 orientToPath:NO	// ノードのZ軸はパスに沿って回転
								 duration:4];
			
			[sprite runAction:action];
		}
		if(_actionNum==3){
			//回転
			//SKAction*	action = [SKAction rotateByAngle:-(M_PI/2*3) duration:1.0];
			SKAction*	action = [SKAction rotateToAngle:-(M_PI/2*3) duration:1.0];
			[sprite runAction:action];
		}
		if(_actionNum==4){
			//スケール
			SKAction*	action = [SKAction scaleBy:2.0 duration:1.0];
			//SKAction*	action = [SKAction scaleTo:2.0 duration:1.0];
			[sprite runAction:action];
		}
		if(_actionNum==5){
			//サイズ
			SKAction*	action;
			//action = [SKAction resizeByWidth:50 height:100 duration:1.0];
			action = [SKAction resizeToWidth:50 height:100 duration:1.0];
			[sprite runAction:action];
		}
		if(_actionNum==6){
			//カラー
			SKAction*	action;
			action = [SKAction colorizeWithColor:[UIColor colorWithRed:1 green:0 blue:0 alpha:1]
								colorBlendFactor:1.0
										duration:1.0];
			
			[sprite runAction:action];
		}
		if(_actionNum==7){
			//透明度
			SKAction*	action;
			//action = [SKAction fadeAlphaBy:-0.5 duration:1.0];
			action = [SKAction fadeAlphaTo:0.5 duration:1.0];
			[sprite runAction:action];
		}
		if(_actionNum==8){
			//テクスチャ
			SKAction*	action;
			NSArray* ary = @[[SKTexture textureWithImageNamed:@"Spaceship_1"],
							 [SKTexture textureWithImageNamed:@"Spaceship_2"],
							 [SKTexture textureWithImageNamed:@"Spaceship_3"]];
			action = [SKAction animateWithTextures:ary timePerFrame:1.0];
			/*
			action = [SKAction animateWithTextures:ary
									  timePerFrame:1.0
											resize:NO	// YES=テクスチャのサイズに合わせてリサイズ
										   restore:NO];	// YES=元にもどす
			*/
			[sprite runAction:action];
		}
		
		
		
		//------------------------------
		// アクションの平行実行と連続実行
		//------------------------------
		if(_actionNum==10) {
			//アクションの平行実行
			NSArray* ary = @[[SKAction moveByX:100 y:50 duration:1.0],
							 [SKAction rotateByAngle:M_PI/2 duration:1.0],
							 [SKAction scaleBy:2.0 duration:1.0]];
			SKAction*	action = [SKAction group:ary];
			[sprite runAction:action];
			
		}
		if(_actionNum==11) {
			//アクションの連続実行
			NSArray* ary = @[[SKAction moveByX:100 y:50 duration:1.0],
							 [SKAction waitForDuration:2.0],				//待機
							 [SKAction rotateByAngle:M_PI/2 duration:1.0],
							 [SKAction waitForDuration:2.5 withRange:5.0],	//ランダム待機
							 [SKAction scaleBy:2.0 duration:1.0]];
			SKAction*	action = [SKAction sequence:ary];
			[sprite runAction:action];
			
		}
		if(_actionNum==12){
			//逆アクション
			SKAction*	action;
			action = [[SKAction scaleBy:2.0 duration:1.0] reversedAction];
			[sprite runAction:action];
		}
		
		
		
		//------------------------------
		// カスタムアクション
		//------------------------------
		if(_actionNum==13){
			//子ノードにアクション実行
			SKAction*	rotateAction1 = [SKAction rotateByAngle:M_PI duration:0.5];
			SKAction *action1 = [SKAction runAction:[SKAction repeatActionForever:rotateAction1]
									onChildWithName:@"star1"];
			[sprite runAction:action1];
			
			SKAction*	rotateAction2 = [SKAction rotateByAngle:M_PI duration:2];
			SKAction *action2 = [SKAction runAction:[SKAction repeatActionForever:rotateAction2]
									onChildWithName:@"star2"];
			[sprite runAction:action2];
		}
		if(_actionNum==14){
			//ブロックスを呼ぶ
			SKAction*	action;
			action = [SKAction runBlock:^{
				//無限の回転
				SKAction *action = [SKAction rotateByAngle:M_PI duration:1];
				[sprite runAction:[SKAction repeatActionForever:action]];
			}];
			[sprite runAction:action];
		}
		if(_actionNum==15){
			//メソッド実行
			SKAction*	action;
			action = [SKAction performSelector:@selector(callMethod) onTarget:self];
			[sprite runAction:action];
		}
		if(_actionNum==16){
			//繰り返し実行
			SKAction *action;
			action = [SKAction customActionWithDuration:5.0
											actionBlock:^(SKNode *node, CGFloat elapsedTime) {
												NSLog(@"%f", elapsedTime);
												if(elapsedTime >= 5.0){
													//半透明にする
													SKAction*	action = [SKAction fadeAlphaTo:0.5 duration:1.0];
													[node runAction:action];
												}
											}];
			[sprite runAction:action];
		}
		if(_actionNum==17){
			//回転後にノードを削除
			SKAction *rotateAction = [SKAction rotateByAngle:M_PI duration:1];
			NSArray* ary = @[[SKAction repeatAction:rotateAction count:2],	//回転アクションを２回実行
							 [SKAction removeFromParent]];					//ノードを削除するアクション
			SKAction *action = [SKAction sequence:ary];
			[sprite runAction:action];
		}
    }
}

-(void)update:(CFTimeInterval)currentTime {
    /* Called before each frame is rendered */
}


-(void)callMethod
{
	//ノードを検索
	[self enumerateChildNodesWithName:@"Spaceship" usingBlock:^(SKNode *node, BOOL *stop) {
        if (node.position.y < 200){
			//４５度回転させる
			SKAction *action = [SKAction rotateByAngle:M_PI/4 duration:1];
			[node runAction:action];
		}
	}];
	
}


@end
